/*******************************************************************
 *
 *	File:		EGnuChessBoardCtl.h
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/


#ifndef __E_GNU_CHESS_BOARDCTL_H
#define __E_GNU_CHESS_BOARDCTL_H

#include <coecntrl.h>
#include <coeccntx.h>

#include "EGnuChessModelObserver.h"
#include "EGnuChessModel.h"
#include "EGnuChessMiscCtls.h"

CWsBitmap* LoadNonSharableBitmapL(const TDesC& aFileName, TInt aId);


/*
 * Right, here's what goes on:
 * - CEGnuChessBoardControlBase offers the common chess board draw routines
 *   It uses a TChessBitmaps object for the pieces/pattern bitmaps it need
 *   to draw. CEGnuChessBoardControlBase nor TChessBitmaps, own the bitmaps,
 *   it is assumed that some other object does.
 * 
 * - CEGnuChessBoardControl: implements the 8x8 chess board used in the flip open
 *   and flip closed views. It owns the bitmaps by means of calling 
 *   TChessBitmaps::LoadAllBitmapsL() and TChessBitmaps::DestroyAllBitmaps()
 *
 * - CEGnuChessBoardSingleSquare derives from CEGnuChessBoardControlBase and
 *   implements a 1x1 "chess board" to be used in various settings dialog.
 *   Its draw method draws a square with an optional piece and optional
 *   highlight.
 *
 */

class TBoardInfo
{
public:
	TInt	iPieceSize;
	TInt	iPieceGap;
	TInt	iSquareFrameSize;
	TInt	iBoardFrameSize;
	TInt	SquareDimension() const	{return iPieceSize + 2*iPieceGap + 2*iSquareFrameSize;}
	TInt	BoardDimension() const	{return 8*SquareDimension() + 2*iBoardFrameSize;}
};

class TChessBitmaps
{
public:
	void LoadAllBitmapsL();
	void DestroyAllBitmaps();
	void ReplacePatterns(TInt aPattern);

	void ReColorBitmaps(const TBoardSettings& aBoardSettings);

	// offset matches enum ChessSquare
	CWsBitmap*		iPieces[12][2];
	CWsBitmap*		iMaskPieces[6][2];
	CWsBitmap*		iPatterns[2];
protected:
	void ReColorPiece(CWsBitmap* aBitmap, const TRgb& aColor, TBool aWhite);
	void ReColorPattern(CWsBitmap* aBitmap, const TRgb& aColor);
	TInt			iPattern;
};

const TInt KMaxPatterns = 16;
class TPatternBitmaps
{
public:
	void LoadAllBitmapsL();
	void DestroyAllBitmaps();

	// offset matches enum ChessSquare
	CWsBitmap*		iPatterns[KMaxPatterns];
};

class CEGnuChessBoardControlBase : public CCoeControl 
{
public:
	~CEGnuChessBoardControlBase(){};
	CEGnuChessBoardControlBase(){};

	void SetBoardSettings(const TBoardSettings& aBoardSettings);

protected:
	void DrawFrame(CBitmapContext& aBitmapContext) const;
	void DrawPiece(CBitmapContext& aBitmapContext, const TSquare& aSquare, const TRect& aRect, TInt aPiece) const;
	void DrawSquare(CBitmapContext& aBitmapContext, const TSquare& aSquare, const TRect& aRect, TInt aPiece) const;
	void SquareDrawBackground(CBitmapContext& aBitmapContext, const TSquare& aSquare, const TRect& aRect) const;
	void SquareDrawHighlight(CBitmapContext& aBitmapContext, const TSquare& aSquare, const TRect& aRect) const;

	void SetBoardInfo(TBool aSmall);

	TBool IsDarkSquare(const TSquare& aSquare) const;
	TRect SquareRect(const TSquare& aSquare) const;
	TSquare Square(const TPoint& aPoint);
	TSquare FlippedSquare(const TSquare& aSquare) const;

	virtual TBool IsFlipped() const {return EFalse;}

	TChessBitmaps		iChessBitmaps;
	TBoardSettings		iBoardSettings;

	TBoardInfo			iBoardInfo;
	TSquare				iHighlights[2];
	TSquare				iSquareKeyboard;
	TSquare				iSquarePreSelect;

	TBool				iSmallBoardSize;
	TBool				iIsConstructed;
};



class CEGnuChessBoardControl :	
		public CEGnuChessBoardControlBase, 
		public MEGnuChessModelBoardObserver,
		public MEGnuChessModelGameObserver
{
public:
	static CEGnuChessBoardControl* NewL(CEGnuChessModel& aModel, CCoeControl& aParent);
	~CEGnuChessBoardControl();
	void SetBoardSize(TBool aSmall);
	void SetBoardSettings(const TBoardSettings& aBoardSettings);

protected:			// implements MEGnuChessModelBoardObserver
	virtual void RedrawL(TBool aFullRedraw);
	virtual void DrawHighlightsL();

protected:			// implements MEGnuChessModelGameObserver
	virtual void UserThinksL();
	virtual void MachineThinksL();
	virtual void DisplayMessageL(const TDesC& /*aMessage*/){};
	virtual void CommentPopUpL(const TDesC& /*aTitle*/, const TDesC& /*aComment*/){};
	virtual void CommentPopDown(){};
	virtual void EditCommentPopUpL(TInt /*aIndex*/, const TDesC& /*aTitle*/, const TDesC& /*aText*/){};
	virtual void ModeHighlightL(){}
		
protected:
	CEGnuChessBoardControl(CEGnuChessModel& aModel);
	void ConstructL(CCoeControl& aParent);

	virtual TBool IsFlipped() const {return ::flipView;}


	void HandlePointerEventL(const TPointerEvent& aPointerEvent);
	TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);

	void Draw(const TRect& aRect) const;
	void DrawBoard(CBitmapContext& aBitmapContext, const TRect& aRect) const;

	void SquareDrawNow(const TSquare& aSquare);
	void ResetUserMove();
	void SelectSquare(const TSquare& aSquare);
	TBool MoveKeyboardSelection(TInt aDx, TInt aDy);
	void SetKeyboardSelectionSquare(const TSquare& aSquare);
	void SetPreSelection(const TSquare& aSquare);

	CEGnuChessModel&	iModel;

	TSquare				iSquarePenDown;
	TBool				iUserMove;
};


class CEGnuChessBoardSingleSquare :	public CEGnuChessBoardControlBase
{
public:
	CEGnuChessBoardSingleSquare(){}
	~CEGnuChessBoardSingleSquare(){}
	void ConstructL(TInt aPiece, TBool aWhiteSquare, TBool aHighLight);
	TBool IsWhiteSquare() { return !IsDarkSquare(iSquare);}
	void SetChessBitmaps(const TChessBitmaps& aChessBitmaps);

protected:
	void Draw(const TRect& aRect) const;
	void HandlePointerEventL(const TPointerEvent& aPointerEvent);

	TInt		iPiece;
	TSquare		iSquare;
};


class CPatternChooserCtl : public CCoeControlContainer, public MCoeControlObserver
{
public:
	~CPatternChooserCtl();
	CPatternChooserCtl();
	void ConstructL(const TBoardSettings& aBoardSettings);

	void SetPattern(TInt aPattern);
	TInt Pattern();

protected:
	virtual void SizeChanged();
	virtual void DoLayoutChildren();
	virtual void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);

	void FocusToPreviewButton(CEGnuChessBoardSingleSquare* aControl);
	TRect FocussedRect(CEGnuChessBoardSingleSquare* aControl) const;
	void DrawFocussedRect(CBitmapContext& aBitmapContext, const TRect& aRect) const;
	void Draw(const TRect& aRect) const;

	CEGnuChessBoardSingleSquare*	iFocussedPreviewButton;
	CEGnuChessBoardSingleSquare*	iSquareButtons[KMaxPatterns*2];
	TPatternBitmaps					iPatternBitmaps[2];

	TInt							iPattern;
};

#endif			/* __E_GNU_CHESS_BOARDCTL_H */
