/*******************************************************************
 *
 *	File:		EGnuChessBoardCtl.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <eikenv.h>
#include <EikAppui.h>
#include <EikApp.h>
#include <QuartzKeys.h>

#include "EGnuChessBoardCtl.h"
#include "EGnuChessModel.h"
#include "debug.h"

#include <EChess.rsg>
#include <eikdialg.h>
#include "EGnuChessApp.hrh"
#include "echess.mbg"


static TInt PromotePawn()
{
	TInt	promo = 0;
	CEikDialog*	dlg= new(ELeave)CEikDialog;
	TInt id = dlg->ExecuteLD(R_PROMOTION_DIALOG);
	switch (id)
	{
		case EEikBidPromoteKing:
			promo = 'k';
			break;
			
		case EEikBidPromoteQueen:
			promo = 'q';
			break;

		case EEikBidPromoteRook:
			promo = 'r';
			break;

		case EEikBidPromoteBishop:
			promo = 'b';
			break;

		case EEikBidPromoteKnight:
			promo = 'n';		// not a typo
			break;
		default:
			break;
	}
	return promo;
}

CWsBitmap* LoadNonSharableBitmapL(const TDesC& aFileName, TInt aId)
{
	CEikonEnv* env = CEikonEnv::Static();

	CWsBitmap* bitmap = new(ELeave) CWsBitmap(env->WsSession());
	TInt ret = bitmap->Load(aFileName, aId, EFalse);
	if (ret)
	{
		delete bitmap;
		User::Leave(ret);
	}
	bitmap->SetSizeInTwips(env->ScreenDevice());
	return bitmap;
}

static void BitmapFileName(TDes& aFilename)
{
	CEikonEnv* env = CEikonEnv::Static();

		/*
		 * The .mbm file is stored in same directory as the .exe
		 * Note that I don't use CEikEnv::CreateBitmapL() which shares bitmaps if loaded 
		 * more than once (it calls CWsBitmap::Load with the sharable flag set).
		 */
	aFilename = ((CEikAppUi*)env->AppUi())->Application()->AppFullName();
	TInt		l = aFilename.Length();
	if (l > 3)
	{
		aFilename[l-1] = 'm';
		aFilename[l-2] = 'b';
		aFilename[l-3] = 'm';
	}
}

void TChessBitmaps::DestroyAllBitmaps()
{
	TInt i;
	for (i=0 ; i<12 ; i++)
	{
		delete iPieces[i][0];
		iPieces[i][0] = NULL;

		delete iPieces[i][1];
		iPieces[i][1] = NULL;
	}
	for (i=0 ; i<6 ; i++)
	{
		delete iMaskPieces[i][0];
		iMaskPieces[i][0] = NULL;

		delete iMaskPieces[i][1];
		iMaskPieces[i][1] = NULL;
	}
	delete iPatterns[0];
	iPatterns[0] = NULL;

	delete iPatterns[1];
	iPatterns[1] = NULL;
}


void TChessBitmaps::LoadAllBitmapsL()
{
	TFileName	bitmapFile;
	BitmapFileName(bitmapFile);

	TInt i;
	for (i=0 ; i<12 ; i++)
	{
		iPieces[i][0] = LoadNonSharableBitmapL(bitmapFile, EMbmEchessP17o + i);
		iPieces[i][1] = LoadNonSharableBitmapL(bitmapFile, EMbmEchessP21o + i);
	}
	for (i=0 ; i<6 ; i++)
	{
		iMaskPieces[i][0] = LoadNonSharableBitmapL(bitmapFile, EMbmEchessP17w + i);
		iMaskPieces[i][1] = LoadNonSharableBitmapL(bitmapFile, EMbmEchessP21w + i);
	}
	iPattern = -1;		// force pattern bitmaps to load
	ReplacePatterns(0);
}

void TChessBitmaps::ReColorPattern(CWsBitmap* aBitmap, const TRgb& aColor)
{
	ASSERT(aBitmap->DisplayMode() == EColor4K);

	TSize	size = aBitmap->SizeInPixels();
	HBufC8*	buf = HBufC8::NewLC(size.iWidth * 4);
	TPtr8	p = buf->Des();
    aBitmap->GetScanLine(p, TPoint(0, 0), size.iWidth, aBitmap->DisplayMode());
	TUint16*	ptr = (TUint16*) buf->Des().Ptr();
	TInt diff = aColor.Color4K() - *ptr;
	for (TInt h=0 ; h<size.iHeight ; h++)
	{
		TPtr8	p = buf->Des();
	    aBitmap->GetScanLine(p, TPoint(0, h), size.iWidth, aBitmap->DisplayMode());
		ptr = (TUint16*) buf->Des().Ptr();
		for (TInt w=0 ; w<size.iWidth ; w++)
		{
			TInt pixel = *ptr + diff;
			if (pixel > 4096)
				pixel -= 4096;
			*ptr++ = (TUint16)pixel;
		}
		aBitmap->SetScanLine(p, h);
	}
	CleanupStack::PopAndDestroy();
}

void TChessBitmaps::ReColorPiece(CWsBitmap* aBitmap, const TRgb& aColor, TBool aWhite)
{
	ASSERT(aBitmap->DisplayMode() == EColor4K);

	TSize	size = aBitmap->SizeInPixels();
	HBufC8*	buf = HBufC8::NewLC(size.iWidth * 4);
	TPtr8	p = buf->Des();
    aBitmap->GetScanLine(p, TPoint(0, 0), size.iWidth, aBitmap->DisplayMode());
	TUint16*	ptr = (TUint16*) buf->Des().Ptr();

		// never allow pure white or black
	TUint16		dstColor = (TUint16) aColor.Color4K();
	if (dstColor == 0)
		dstColor++;
	if (dstColor == 0xfff)
		dstColor--;

	for (TInt h=0 ; h<size.iHeight ; h++)
	{
		TPtr8	p = buf->Des();
	    aBitmap->GetScanLine(p, TPoint(0, h), size.iWidth, aBitmap->DisplayMode());
		ptr = (TUint16*) buf->Des().Ptr();
		for (TInt w=0 ; w<size.iWidth ; w++)
		{
				// for white pieces, we recolor every pixel that isn't pure black
			if (aWhite)
			{
				if (*ptr != (TUint16) 0)
					*ptr = dstColor;
			}
			else
			{
					// for black pieces, we recolor every pixel that isn't pure white
				if (*ptr != (TUint16) 0xfff)
					*ptr = dstColor;
			}
			ptr++;
		}
		aBitmap->SetScanLine(p, h);
	}
	CleanupStack::PopAndDestroy();
}


void TChessBitmaps::ReColorBitmaps(const TBoardSettings& aBoardSettings)
{
	if (iPatterns[0])
		ReColorPattern(iPatterns[0], aBoardSettings.iColorLightSquare);
	if (iPatterns[1])
		ReColorPattern(iPatterns[1], aBoardSettings.iColorDarkSquare);

	TInt i;
	for (i=0 ; i<BlackPawn ; i++)
	{
		if (iPieces[i][0])
			ReColorPiece(iPieces[i][0], aBoardSettings.iColorWhitePiece, ETrue);
		if (iPieces[i][1])
			ReColorPiece(iPieces[i][1], aBoardSettings.iColorWhitePiece, ETrue);
	}
	for (i=BlackPawn ; i<EmptySquare ; i++)
	{
		if (iPieces[i][0])
			ReColorPiece(iPieces[i][0], aBoardSettings.iColorBlackPiece, EFalse);
		if (iPieces[i][1])
			ReColorPiece(iPieces[i][1], aBoardSettings.iColorBlackPiece, EFalse);
	}
}


void TChessBitmaps::ReplacePatterns(TInt aPattern)
{
	ASSERT(aPattern < KMaxPatterns);

	if (iPattern != aPattern)
	{
		TFileName	bitmapFile;
		BitmapFileName(bitmapFile);

		if ((aPattern<0) || (aPattern>=KMaxPatterns))
			aPattern = 0;
			
		CWsBitmap* bitmap0 = NULL;
		CWsBitmap* bitmap1 = NULL;

		TRAPD(err0, bitmap0 = LoadNonSharableBitmapL(bitmapFile, EMbmEchess1 + aPattern));
		TRAPD(err1, bitmap1 = LoadNonSharableBitmapL(bitmapFile, EMbmEchess1 + aPattern));
		if (!err0 && !err1)
		{
			delete iPatterns[0];
			iPatterns[0] = bitmap0;
			delete iPatterns[1];
			iPatterns[1] = bitmap1;

			iPattern = aPattern;
		}
		else
		{
			delete bitmap0;
			delete bitmap1;	// well, this can never happen, added for completeness
		}
	}
}


void TPatternBitmaps::LoadAllBitmapsL()
{
	TFileName	bitmapFile;
	BitmapFileName(bitmapFile);

	for (TInt i=0 ; i<KMaxPatterns ; i++)
		iPatterns[i] = LoadNonSharableBitmapL(bitmapFile, EMbmEchess1+i);
}

void TPatternBitmaps::DestroyAllBitmaps()
{
	for (TInt i=0 ; i<KMaxPatterns ; i++)
	{
		delete iPatterns[i];
		iPatterns[i] = NULL;
	}
}


void CEGnuChessBoardControlBase::SetBoardSettings(const TBoardSettings& aBoardSettings)
{
	iBoardSettings = aBoardSettings;
	iChessBitmaps.ReColorBitmaps(iBoardSettings);
}

void CEGnuChessBoardControlBase::SetBoardInfo(TBool aSmall)
{
	if (aSmall)
	{
		iBoardInfo.iPieceSize		= 17;
		iBoardInfo.iPieceGap		= 0;
		iBoardInfo.iSquareFrameSize	= 0;
		iBoardInfo.iBoardFrameSize	= 1;
	}
	else
	{
		iBoardInfo.iPieceSize		= 21;
		iBoardInfo.iPieceGap		= 2; // 1;
		iBoardInfo.iSquareFrameSize	= 0; //1;
		iBoardInfo.iBoardFrameSize	= 1;
	}
	iSmallBoardSize = aSmall;
}



TBool CEGnuChessBoardControlBase::IsDarkSquare(const TSquare& aSquare) const
{
	return ((aSquare.iRow + aSquare.iCol) & 1) == 0;
}

TSquare CEGnuChessBoardControlBase::FlippedSquare(const TSquare& aSquare) const
{
	return IsFlipped() ? TSquare(7 - aSquare.iRow, 7 - aSquare.iCol) : aSquare;
}

TRect CEGnuChessBoardControlBase::SquareRect(const TSquare& aSquare) const
{
	TSquare		square = FlippedSquare(aSquare);

	TInt squareDim = iBoardInfo.SquareDimension();
	TPoint p(square.iCol * squareDim, (7 - square.iRow) * squareDim);
	p+=TPoint(iBoardInfo.iBoardFrameSize, iBoardInfo.iBoardFrameSize);
	return TRect(p, TSize(squareDim, squareDim));
}



TSquare CEGnuChessBoardControlBase::Square(const TPoint& aPoint)
{
	TPoint	boardPos = aPoint - Position();
	boardPos -= TPoint(iBoardInfo.iBoardFrameSize, iBoardInfo.iBoardFrameSize);

	TInt	squareDim = iBoardInfo.SquareDimension();
	TSquare	s;
	s.Empty();
	s.iCol = boardPos.iX / squareDim;
	s.iRow = 7 - (boardPos.iY / squareDim);
	s.Normalize();

	return FlippedSquare(s);
}


void CEGnuChessBoardControlBase::DrawPiece(CBitmapContext& aBitmapContext, const TSquare& /*aSquare*/, const TRect& aRect, TInt aPiece) const
{
	TInt	o = iSmallBoardSize ? 0 : 1;

	if (aPiece < EmptySquare)
	{
		TInt	gap = iBoardInfo.iSquareFrameSize + iBoardInfo.iPieceGap;
		TPoint	p = aRect.iTl + TPoint(gap, gap);
		TInt mask = (aPiece < BlackPawn) ? aPiece : aPiece - BlackPawn;
		aBitmapContext.BitBltMasked(p, iChessBitmaps.iPieces[aPiece][o], TRect(TSize(iBoardInfo.iPieceSize, iBoardInfo.iPieceSize)), 
			iChessBitmaps.iMaskPieces[mask][o], ETrue);
	}
}

void CEGnuChessBoardControlBase::SquareDrawBackground(CBitmapContext& aBitmapContext, const TSquare& aSquare, const TRect& aRect) const
{
	TInt dim = iBoardInfo.SquareDimension();
	TRect	rect(TSize(dim, dim));
	aBitmapContext.Reset();
	aBitmapContext.BitBlt(aRect.iTl, IsDarkSquare(aSquare) ? iChessBitmaps.iPatterns[1] : iChessBitmaps.iPatterns[0], rect);
}

void CEGnuChessBoardControlBase::SquareDrawHighlight(CBitmapContext& aBitmapContext, const TSquare& aSquare, const TRect& aRect) const
{
	TBool  preSelection = ((iSmallBoardSize && (aSquare == iSquareKeyboard)) || (aSquare == iSquarePreSelect));

	if ((aSquare == iHighlights[0]) || (aSquare == iHighlights[1]) || preSelection)
	{
		TInt	penSize = iBoardInfo.iSquareFrameSize ? iBoardInfo.iSquareFrameSize : 1;
		aBitmapContext.Reset();
		aBitmapContext.SetBrushStyle(CGraphicsContext::ENullBrush);
		aBitmapContext.SetPenColor(iBoardSettings.iColorHighlight);
		aBitmapContext.SetPenSize(TSize(penSize, penSize));
		if (preSelection)
			aBitmapContext.SetPenStyle(CGraphicsContext::EDashedPen);
		aBitmapContext.DrawRect(aRect);
	}
}

void CEGnuChessBoardControlBase::DrawSquare(CBitmapContext& aBitmapContext, const TSquare& aSquare, const TRect& aRect, TInt aPiece) const
{
	SquareDrawBackground(aBitmapContext, aSquare, aRect);
	DrawPiece(aBitmapContext, aSquare, aRect, aPiece);
	SquareDrawHighlight(aBitmapContext, aSquare, aRect);
}

void CEGnuChessBoardControlBase::DrawFrame(CBitmapContext& aBitmapContext) const
{
	if (iBoardInfo.iBoardFrameSize)
	{
		aBitmapContext.Reset();
		aBitmapContext.SetPenColor(KRgbBlack);
		aBitmapContext.SetPenSize(TSize(iBoardInfo.iBoardFrameSize, iBoardInfo.iBoardFrameSize));

		TInt	boardWidth = iBoardInfo.BoardDimension();
		TRect rect(Position(), TSize(boardWidth, boardWidth));
		aBitmapContext.DrawRect(rect);
	}
}


/*****************************************
 *
 *
 *
 *****************************************/

CEGnuChessBoardControl::CEGnuChessBoardControl(CEGnuChessModel& aModel) : iModel(aModel)
{
}

CEGnuChessBoardControl::~CEGnuChessBoardControl()
{
	static_cast<CEikAppUi*>(iEikonEnv->AppUi())->RemoveFromStack(this);
	iChessBitmaps.DestroyAllBitmaps();
}


CEGnuChessBoardControl* CEGnuChessBoardControl::NewL(CEGnuChessModel& aModel, CCoeControl& aParent)
{
	CEGnuChessBoardControl*	self = new(ELeave)CEGnuChessBoardControl(aModel);
	CleanupStack::PushL(self);
	self->ConstructL(aParent);
	CleanupStack::Pop();		// self
	return self;
}


void CEGnuChessBoardControl::ConstructL(CCoeControl& aParent)
{
	SetContainerWindowL(aParent);
	Window().SetPointerGrab(ETrue);
	Window().PointerFilter(EPointerFilterDrag, 0);

	iChessBitmaps.LoadAllBitmapsL();
	static_cast<CEikAppUi*>(iEikonEnv->AppUi())->AddToStackL(this);

	iModel.AddModelBoardObserver(*this);
	iModel.AddModelGameObserver(*this);

	iBoardSettings.Defaults();
	CEGnuChessBoardControlBase::SetBoardSettings(iBoardSettings);

	SetBoardSize(EFalse);		// construct large board
}

void CEGnuChessBoardControl::SetBoardSettings(const TBoardSettings& aBoardSettings)
{
	iChessBitmaps.ReplacePatterns(aBoardSettings.iPattern);
	CEGnuChessBoardControlBase::SetBoardSettings(aBoardSettings);
}


void CEGnuChessBoardControl::SetBoardSize(TBool aSmall)
{
	if ((iSmallBoardSize != aSmall) || !iIsConstructed)
	{
		SetBoardInfo(aSmall);	// sets iSmallBoardSize
		SetSize(TSize(iBoardInfo.BoardDimension(), iBoardInfo.BoardDimension()));
		iIsConstructed = ETrue;
	}
}


void CEGnuChessBoardControl::SelectSquare(const TSquare& aSquare)
{
	if (!iUserMove)
	{
		if (iModel.IsOKToStartUserMove(aSquare))
		{
			iModel.SetHighlight(aSquare);
			iUserMove = ETrue;

			iSquareKeyboard = aSquare;
			if (!MoveKeyboardSelection(1, 0))
				if (!MoveKeyboardSelection(-1, 0))
					if (!MoveKeyboardSelection(0, 1))
						if (!MoveKeyboardSelection(0, -1))
							ASSERT(EFalse);
		}
		else
		{
			SetKeyboardSelectionSquare((gameMode == MachinePlaysWhite) ? TSquare(7, 7) : TSquare(0, 0));
		}
	}
	else
	{
		if (aSquare == iHighlights[0])
		{
			ResetUserMove();
			iModel.ClearHighlightsL();
		}
		else
		{
			TInt	promo = 0;
			if (iModel.IsPromotion(aSquare))
				promo = PromotePawn();
			iModel.UserMoveEvent(aSquare, promo);

				// if it's still our turn, the move was illegal
			if (iModel.iUserThinks)
			{
				ResetUserMove();
				iModel.ClearHighlightsL();
			}
			else
				SetKeyboardSelectionSquare(TSquare(-1, -1));
		}
	}
}

void CEGnuChessBoardControl::SetPreSelection(const TSquare& aSquare)
{
	TSquare	oldSquarePreSelect = iSquarePreSelect;
	iSquarePreSelect = aSquare;
	SquareDrawNow(iSquarePreSelect);
	SquareDrawNow(oldSquarePreSelect);
}
	
void CEGnuChessBoardControl::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
	if (!iModel.iUserThinks)
		return;
	
	if (aPointerEvent.iType == TPointerEvent::EButton1Down)
	{
		iSquarePenDown = Square(aPointerEvent.iPosition);
		SetPreSelection(iSquarePenDown);
	}
	else if (aPointerEvent.iType == TPointerEvent::EDrag)
	{
		TSquare square = Square(aPointerEvent.iPosition);
		if (square != iSquarePenDown)
		{
			iSquarePenDown = square;
			SetPreSelection(iSquarePenDown);
		}
	}
	else if (aPointerEvent.iType == TPointerEvent::EButton1Up)
	{
		TSquare	square = Square(aPointerEvent.iPosition);
		SetPreSelection(TSquare());
		if (square == iSquarePenDown)
			SelectSquare(iSquarePenDown);
	}
}


TBool CEGnuChessBoardControl::MoveKeyboardSelection(TInt aDx, TInt aDy)
{
	TBool	hasMoved = EFalse;
	TSquare square(iSquareKeyboard.iRow + aDx, iSquareKeyboard.iCol + aDy);
	square.Normalize();
	if (!square.IsEmpty())
	{
		SetKeyboardSelectionSquare(square);
		hasMoved = ETrue;
	}
	return hasMoved;
}


TKeyResponse CEGnuChessBoardControl::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
{
	TKeyResponse keyResponse = EKeyWasNotConsumed;

	if (iSmallBoardSize && iModel.iUserThinks)
	{
		if (aType == EEventKey)
		{
			DPRINT((HERE("key=0x%x"), aKeyEvent.iCode));
			switch(aKeyEvent.iCode)
			{
				case EQuartzKeyTwoWayUp:
					MoveKeyboardSelection(1, 0);
					keyResponse = EKeyWasConsumed;
					break;
				case EQuartzKeyTwoWayDown:
					MoveKeyboardSelection(-1, 0);
					keyResponse = EKeyWasConsumed;
					break;
				case EKeyDevice3:
					MoveKeyboardSelection(0, -1);
					keyResponse = EKeyWasConsumed;
					break;
				case EQuartzKeyConfirm:
					SelectSquare(iSquareKeyboard);
					keyResponse = EKeyWasConsumed;
					break;
				default:
					break;
			}
		}
		else if (aType == EEventKeyUp)
		{
			switch(aKeyEvent.iScanCode)
			{
				case EStdKeyDeviceD:
					MoveKeyboardSelection(0, 1);
					keyResponse = EKeyWasConsumed;
					break;
				default:
					break;
			}
		}
	}

	return keyResponse;
}


void CEGnuChessBoardControl::DrawBoard(CBitmapContext& aBitmapContext, const TRect& aRect) const
{
	TPoint  p = Position();
	for(TInt row=0 ; row<8 ; row++)
	{
		for(TInt col=0 ; col<8 ; col++)
		{
			TSquare		square(row, col);
			TRect		r = SquareRect(square);
			r.Move(p);
			if (aRect.Intersects(r))
				DrawSquare(aBitmapContext, square, r, iModel.iBoard[square.iRow][square.iCol]);
		}
	}		
}

void CEGnuChessBoardControl::Draw(const TRect& aRect) const
{
    CWindowGc& gc=SystemGc();
	DrawFrame(gc);
	DrawBoard(gc, aRect);
}

void CEGnuChessBoardControl::ResetUserMove()
{
	SetKeyboardSelectionSquare((gameMode == MachinePlaysWhite) ? TSquare(7, 7) : TSquare(0, 0));
	SetPreSelection(TSquare());
	iUserMove = EFalse;
}

void CEGnuChessBoardControl::SquareDrawNow(const TSquare& aSquare)
{
	if (aSquare.IsEmpty())
		return;
	TRect rect= SquareRect(aSquare);
	TRect drawRect(rect.iTl + Position(), rect.Size());

	Window().Invalidate(drawRect);
	ActivateGc();
	Window().BeginRedraw(drawRect);
	DrawSquare(SystemGc(), aSquare, drawRect, iModel.iBoard[aSquare.iRow][aSquare.iCol]);
	Window().EndRedraw();
	DeactivateGc();
	iEikonEnv->WsSession().Flush();
}



/*************************************************
 *
 * MEGnuChessModelBoardObserver implementation
 *
 *************************************************/

void CEGnuChessBoardControl::DrawHighlightsL()
{
	TSquare		highLight0 = iHighlights[0];
	TSquare		highLight1 = iHighlights[1];
	iHighlights[0] = iModel.iHighlights[0];
	iHighlights[1] = iModel.iHighlights[1];
	SquareDrawNow(iHighlights[0]);
	SquareDrawNow(iHighlights[1]);
	if (highLight0 != iHighlights[0])
		SquareDrawNow(highLight0);
	if (highLight1 != iHighlights[1])
		SquareDrawNow(highLight1);
}

void CEGnuChessBoardControl::RedrawL(TBool aFullRedraw)
{
	TBoard&	board = iModel.iBoard;
	TBoard&	prevBoard = iModel.iPrevBoard;
	for(TInt row=0 ; row<8 ; row++)
	{
		for(TInt col=0 ; col<8 ; col++)
		{
			if ((board[row][col] != prevBoard[row][col]) || aFullRedraw)
				SquareDrawNow(TSquare(row, col));
		}
	}		
}

void CEGnuChessBoardControl::UserThinksL()
{
	ResetUserMove();
}

void CEGnuChessBoardControl::MachineThinksL()
{
	SetKeyboardSelectionSquare(TSquare());
}

void CEGnuChessBoardControl::SetKeyboardSelectionSquare(const TSquare& aSquare)
{
	TSquare		oldSquare = iSquareKeyboard;
	iSquareKeyboard = aSquare;
	if (iSmallBoardSize)
	{
		SquareDrawNow(iSquareKeyboard);
		SquareDrawNow(oldSquare);
	}
}

/***********************************************
 *
 * CEGnuChessBoardSingleSquare
 *
 ***********************************************/


void CEGnuChessBoardSingleSquare::ConstructL(TInt aPiece, TBool aWhiteSquare, TBool aHighLight)
{
	iPiece = aPiece;
	iSquare = (aWhiteSquare) ? TSquare(0, 1) : TSquare(0, 0);

	SetBoardInfo(EFalse);
	TInt	size = iBoardInfo.SquareDimension();
	SetSize(TSize(size, size));

	if (aHighLight)
		iHighlights[0] = iSquare;
}

void CEGnuChessBoardSingleSquare::SetChessBitmaps(const TChessBitmaps& aChessBitmaps)
{
	iChessBitmaps = aChessBitmaps;
}

void CEGnuChessBoardSingleSquare::Draw(const TRect& /*aRect*/) const
{
    CWindowGc& gc=SystemGc();
	DrawSquare(gc, iSquare, Rect(), iPiece);
	SquareDrawHighlight(gc, iSquare, Rect());
}

void CEGnuChessBoardSingleSquare::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
	if (aPointerEvent.iType == TPointerEvent::EButton1Up)
	{
		if (Rect().Contains(aPointerEvent.iPosition))
			ReportEventL(MCoeControlObserver::EEventStateChanged);
	}
}


/***********************************************************
 *
 * CPatternChooserCtl
 *
 ***********************************************************/

void CPatternChooserCtl::SizeChanged()
{
	LayoutChildren();
}



#define KSquareButtonHorGap		10
#define KSquareButtonVerGap		10
#define KSquareButtonsPerRow	3

void CPatternChooserCtl::DoLayoutChildren()
{
	TPoint	topLeftPoint = Position();
	TSize	size(25, 25);

	topLeftPoint.iX += (Size().iWidth - (KSquareButtonsPerRow * size.iWidth *2) - ((KSquareButtonsPerRow-1) * KSquareButtonHorGap * 2)) / 2;
	TPoint	p = topLeftPoint;

	TInt rows = (KMaxPatterns + KSquareButtonsPerRow -1)/ KSquareButtonsPerRow;
	TInt pattern = 0;
	for (TInt i=0 ; i<rows ; i++)
	{
		for (TInt j=0 ; j<KSquareButtonsPerRow ; j++)
		{
			if (pattern >= KMaxPatterns * 2)
				break;
			iSquareButtons[pattern++]->SetRect(TRect(p, size));
			p.iX += size.iWidth;
			iSquareButtons[pattern++]->SetRect(TRect(p, size));
			p.iX += size.iWidth + KSquareButtonHorGap;
		}
		p.iX = topLeftPoint.iX;
		p.iY += size.iHeight + KSquareButtonVerGap;
	}
}


CPatternChooserCtl::CPatternChooserCtl()
{
}

CPatternChooserCtl::~CPatternChooserCtl()
{
	DestroyChildren();

	iPatternBitmaps[0].DestroyAllBitmaps();
	iPatternBitmaps[1].DestroyAllBitmaps();
}

void CPatternChooserCtl::ConstructL(const TBoardSettings& aBoardSettings)
{
	CCoeControlContainer::ConstructL();
//	Window().PointerFilter(EPointerFilterDrag, 0);

		// load bitmaps twice, once for the light and once for the dark squares
	iPatternBitmaps[0].LoadAllBitmapsL();
	iPatternBitmaps[1].LoadAllBitmapsL();

	TBool	white=ETrue;
	TChessBitmaps	chessBitmaps;
	Mem::FillZ(&chessBitmaps, sizeof(chessBitmaps));
	for (TInt i=0 ; i<KMaxPatterns*2 ; i++)
	{
		if (white)
		{
			chessBitmaps.iPatterns[0] = iPatternBitmaps[i % 2].iPatterns[i/2];
			chessBitmaps.iPatterns[1] = NULL;
		}
		else
		{
			chessBitmaps.iPatterns[0] = NULL;
			chessBitmaps.iPatterns[1] = iPatternBitmaps[i % 2].iPatterns[i/2];
		}

		iSquareButtons[i] = new(ELeave)CEGnuChessBoardSingleSquare;
		iSquareButtons[i]->SetContainerWindowL(*this);
		iSquareButtons[i]->ConstructL(EmptySquare, white, EFalse);
		iSquareButtons[i]->SetChessBitmaps(chessBitmaps);
		iSquareButtons[i]->SetBoardSettings(aBoardSettings);
		iSquareButtons[i]->SetObserver(this);
		AddChildL(iSquareButtons[i]);	// do not blank child in redraw
		white = !white;
	}
}


void CPatternChooserCtl::HandleControlEventL(CCoeControl* aControl, TCoeEvent aEventType)
{
	if (aEventType == EEventStateChanged)
	{
		FocusToPreviewButton((CEGnuChessBoardSingleSquare*)aControl);
	}
}


TRect CPatternChooserCtl::FocussedRect(CEGnuChessBoardSingleSquare* aControl) const
{
	TRect rect = aControl->Rect();
	if (aControl->IsWhiteSquare())
		rect.iBr.iX += aControl->Size().iWidth;
	else
		rect.iTl.iX -= aControl->Size().iWidth;

	rect.Grow(TSize(2, 2));
	return rect;
}

void CPatternChooserCtl::FocusToPreviewButton(CEGnuChessBoardSingleSquare* aControl)
{
	for (TInt i=0 ; i<KMaxPatterns * 2 ; i++)
	{
		if (iSquareButtons[i] == aControl)
		{
			iPattern = i/2;

			if (iFocussedPreviewButton != aControl)
			{
				Window().Invalidate(FocussedRect(iFocussedPreviewButton));
		
				iFocussedPreviewButton = aControl;
				Window().Invalidate(FocussedRect(iFocussedPreviewButton));
			}
		}
	}
}

void CPatternChooserCtl::DrawFocussedRect(CBitmapContext& aBitmapContext, const TRect& aRect) const
{
	aBitmapContext.Reset();
	aBitmapContext.SetBrushStyle(CGraphicsContext::ENullBrush);
	aBitmapContext.SetPenStyle(CGraphicsContext::EDashedPen);
	aBitmapContext.SetPenSize(TSize(1, 1));
	aBitmapContext.SetPenColor(KRgbBlack);
	aBitmapContext.DrawRect(aRect);
}

void CPatternChooserCtl::Draw(const TRect& aRect) const
{
	CCoeControlContainer::Draw(aRect);

	CWindowGc& gc = SystemGc();
	DrawFocussedRect(gc, FocussedRect(iFocussedPreviewButton));
}

void CPatternChooserCtl::SetPattern(TInt aPattern)
{
	iPattern = aPattern;
	iFocussedPreviewButton = iSquareButtons[aPattern * 2];
}

TInt CPatternChooserCtl::Pattern()
{
	return iPattern;
}
