/*******************************************************************
 *
 *	File:		EGnuChessEngConnection.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <e32std.h>
#include <eikdll.h>
#include <in_sock.h>
#include "EGnuChessEngConnection.h"
#include "debug.h"

#ifdef __WINS__
#define EXE_NAME_ON_C	_L("CRAFTY.DLL")
#define EXE_NAME_ON_D	_L("CRAFTY.DLL")
#else
#define EXE_NAME_ON_C	_L("c:\\system\\programs\\CRAFTY.EXE")
#define EXE_NAME_ON_D	_L("d:\\system\\programs\\CRAFTY.EXE")
#endif


CEGnuChessEngConnection::~CEGnuChessEngConnection()
{
	Cancel();
	if (iServerSocket.SubSessionHandle()>0)
		iServerSocket.Close();
	if (iSocket.SubSessionHandle()>0)
		iSocket.Close();
	if (iSockSrvSession.Handle()>0)
		iSockSrvSession.Close();
	/*
	 * Don't stop engine. Our socket-end is closed and the engine will pick up that.
	 */
//	EngineStop();
}

CEGnuChessEngConnection::CEGnuChessEngConnection() : CActive(EPriorityStandard)
{
}

CEGnuChessEngConnection* CEGnuChessEngConnection::NewL()
{
	CEGnuChessEngConnection*	self = new(ELeave)CEGnuChessEngConnection;
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();		// self
	return self;
}

void CEGnuChessEngConnection::ConstructL()
{
	CActiveScheduler::Add(this);
	User::LeaveIfError(iSockSrvSession.Connect());
	StartAndWaitForEngineToConnectL();
	EngineReceiveData();
}


void CEGnuChessEngConnection::SetEngineDataProcessor(MEGnuChessEngDataProcessor* aEGnuChessEngDataProcessor)
{
	iEGnuChessEngDataProcessor = aEGnuChessEngDataProcessor;
}


void CEGnuChessEngConnection::StartAndWaitForEngineToConnectL()
{
	TInetAddr	serv_addr(KInetAddrLoop, 6543);

	User::LeaveIfError(iSocket.Open(iSockSrvSession));
	User::LeaveIfError(iServerSocket.Open(iSockSrvSession, KAfInet, KSockStream, KProtocolInetTcp));
	User::LeaveIfError(iServerSocket.Bind(serv_addr));
	User::LeaveIfError(iServerSocket.Listen(5));

	// now that the necessary EPOC servers and libraries are loaded, start our chess server...
	EngineStartL();

		/*
		 * This is a bit naughty...
		 * The backend layer wants to wait until the connection is made.
		 * We can't call Accept() synchronously because that may block us forever.
		 * Instead we issue it asynchronously and wait a while for it to complete.
		 *
		 * Note that it is imperitive to call User::WaitForRequest at the end. 
		 * Although it appears to make no sense because the request is already completed,
		 * it is absolutely necessary because we need to co-operate with the active scheduler. 
		 */
	TRequestStatus	status = KRequestPending;
	iServerSocket.Accept(iSocket, status);
	for(TInt i=0 ; (status.Int() == KRequestPending) && (i<10) ; i++)
		User::After(500 * 1000);
	if (status.Int() == KRequestPending)
	{
		iServerSocket.CancelAccept();
		status = KErrServerTerminated;	// whatever
	}
	User::WaitForRequest(status);

	User::LeaveIfError(status.Int());

	DPRINT((HERE("******************************")));
	DPRINT((HERE("ENGINE CONNECTED")));
	DPRINT((HERE("******************************")));
}

void CEGnuChessEngConnection::EngineStop()
{
	RThread	thread;
	if (thread.Open(iChessEngineThread) == KErrNone)
	{
		thread.Terminate(KErrNone);
		thread.Close();
	}
}

void CEGnuChessEngConnection::EngineStartL()
{
	TRAPD(err, iChessEngineThread = EikDll::StartExeL(EXE_NAME_ON_C));
	if (err)
		iChessEngineThread = EikDll::StartExeL(EXE_NAME_ON_D);
}

TInt CEGnuChessEngConnection::EngineWriteData(const TDesC8& aData)
{
//	RDebug::Print(_L("ECHESS SENDS: -%S-"), &CONV_TDES8_UNI_DES8(aData));
	TRequestStatus	status;
	iSocket.Send(aData, 0, status);
	User::WaitForRequest(status);
	return status.Int();
}


void CEGnuChessEngConnection::EngineReceiveData()
{
	iRawRecvBuf.SetLength(0);
	iSocket.RecvOneOrMore(iRawRecvBuf, 0, iStatus, iRawRecvBufLen);
	SetActive();
}


void CEGnuChessEngConnection::EngineProcessData()
{
	TPtrC8		p = iRawRecvBuf;
	TBool		done = EFalse;

	DPRINT((_L("Engine spits out RAW: -%S-"), &CONV_TDES8_UNI_DES8(iRawRecvBuf)));
	while(!done)
	{
		TInt pos = p.Locate((TChar('\n')));
		if (pos > 0)
		{
			TInt len = p.Length();
			if (len + pos <= KMaxRecvBuf)
			{
				iRecvBuf = p.Left(pos+1);
				DPRINT((_L("Engine spits out: %S"), &CONV_TDES8_UNI_DES8(iRecvBuf)));
				if (iEGnuChessEngDataProcessor)
					iEGnuChessEngDataProcessor->ProcessData(iRecvBuf);
				if (pos != len - 1)
				{
					p.Set(p.Right(len - pos - 1));
				}
				else
					done = ETrue;
			}
			else
			{
				ASSERT(EFalse);
				iRecvBuf.SetLength(0);
			}
		}
		else
		{
			iRecvBuf.Append(p);
			DPRINT((_L("Engine spits out: %S"), &CONV_TDES8_UNI_DES8(iRecvBuf)));
			done = ETrue;
		}
	}
}

void CEGnuChessEngConnection::RunL()
{
	if (iStatus.Int() == KErrNone)
	{
		EngineProcessData();
	}
	EngineReceiveData();
}

void CEGnuChessEngConnection::DoCancel()
{
	iSocket.CancelRecv();
}
