/*******************************************************************
 *
 *	File:		EGnuChessFrontend.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include <e32std.h>
#include <string.h>
#include "debug.h"
#include "EGnuChessFrontendProcessor.h"
#include "EGnuChessEngConnection.h"
#include "EGnuChessApp.hrh"

#include "winboard.h"

#ifdef __WINS__
// most API calls are not supported, these warnings only clutter the build output
#pragma warning (disable:4100)		// unreferenced formal parameter
#endif

static TBuf<KMaxEditSize>	sTextBufer;
static TBuf<KMaxEditSize>	sTextBufer1;


/******************************************
 *
 * Front end processor
 *
 ******************************************/

static MFrontendProcessor* sFrontendProcessor = NULL;
void SetFrontendProcessor(MFrontendProcessor* aFrontendProcessor)
{
	sFrontendProcessor = aFrontendProcessor;
}


/******************************************
 *
 * Child process
 *
 ******************************************/

class CChildProcBase : public CBase
{
public:
	virtual void DestroyChildProcess(int signal) = 0;
	virtual void InterruptChildProcess() = 0;
	virtual void AddInputSource(TInt aLineByLine, InputCallback aFunc, VOIDSTAR aClosure) = 0;
	virtual void RemoveInputSource() = 0;
	virtual int OutputToProcessDelayed(TText8* aMessage, TInt aCount, TInt aMsdelay=0) = 0;
};

class CChildProcChessEngine : public CChildProcBase, public MEGnuChessEngDataProcessor
{
public:
	~CChildProcChessEngine();
	static CChildProcChessEngine* NewL();

public:		// from CChildProcBase
	virtual void DestroyChildProcess(int signal);
	virtual void InterruptChildProcess();
	virtual void AddInputSource(TInt aLineByLine, InputCallback aFunc, VOIDSTAR aClosure);
	virtual void RemoveInputSource();
	virtual int OutputToProcessDelayed(TText8* aMessage, TInt aCount, TInt aMsdelay=0);

public:		// implements MEGnuChessEngDataProcessor
	virtual void ProcessData(const TDesC8& aData);

protected:
	CChildProcChessEngine(){}
	void ConstructL();

	TInt			iLineByLine;
	InputCallback	iFunc;
	VOIDSTAR		iClosure;
	RSemaphore		iSignalSemaphore;

	CEGnuChessEngConnection*	iEGnuChessEngConnection;
};

CChildProcChessEngine::~CChildProcChessEngine()
{
	if (iSignalSemaphore.Handle())
		iSignalSemaphore.Close();
	delete iEGnuChessEngConnection;
}

CChildProcChessEngine* CChildProcChessEngine::NewL()
{
	CChildProcChessEngine* self = new(ELeave) CChildProcChessEngine;
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();
	return self;
}

void CChildProcChessEngine::ConstructL()
{
	User::LeaveIfError(iSignalSemaphore.CreateGlobal(_L("CHESS_SIG"), 0));

	iEGnuChessEngConnection = CEGnuChessEngConnection::NewL();
}

void CChildProcChessEngine::DestroyChildProcess(int signal)
{
}

void CChildProcChessEngine::InterruptChildProcess()
{
	iSignalSemaphore.Signal();
}

void CChildProcChessEngine::AddInputSource(TInt aLineByLine, InputCallback aFunc, VOIDSTAR aClosure)
{
	iLineByLine = aLineByLine;
	iFunc = aFunc;
	iClosure = aClosure;
	iEGnuChessEngConnection->SetEngineDataProcessor(this);
}

void CChildProcChessEngine::ProcessData(const TDesC8& aData)
{
	this->iFunc((InputSourceRef) this, iClosure, (char*) aData.Ptr(), aData.Length(), 0);
}

void CChildProcChessEngine::RemoveInputSource()
{
	iEGnuChessEngConnection->SetEngineDataProcessor(NULL);
}

int CChildProcChessEngine::OutputToProcessDelayed(TText8* aMessage, TInt aCount, TInt aMsdelay)
{
	TPtrC8	des(aMessage, aCount);
	return iEGnuChessEngConnection->EngineWriteData(des);
}



/******************************************
 *
 * Front implementation
 *
 ******************************************/


void ModeHighlight P((void))
{
	static TInt	prevPausing = 0;
	DPRINT((HERE("FEND: ModeHighlight")));
	if (prevPausing != pausing)
	{
		DisplayMessage(pausing ? "Game paused" : "Game resumed", "");
		prevPausing = pausing;
	}
	sFrontendProcessor->ModeHighlightL();
}


void SetICSMode P((void))
{
	DEBUGGER();
}

void SetGNUMode P((void))
{
	DEBUGGER();
}

void SetNCPMode P((void))
{
	DEBUGGER();
}

void SetCmailMode P((void))
{
	DEBUGGER();
}

void SetTrainingModeOn P((void))
{
	DEBUGGER();
}

void SetTrainingModeOff P((void))
{
	DEBUGGER();
}

void SetUserThinkingEnables P((void))
{
	DPRINT((HERE("FEND: SetUserThinkingEnables - TODO (menu opoptions)")));
	sFrontendProcessor->UserThinksL();
}

void SetMachineThinkingEnables P((void))
{
	DPRINT((HERE("FEND: SetMachineThinkingEnables - TODO (menu opoptions)")));
	sFrontendProcessor->MachineThinksL();
}

void DisplayTitle P((String title))
{
	DPRINT((HERE("FEND: DisplayTitle: %S"), &CONV_UNI(title)));
}

void DisplayMessage P((String message, String extMessage))
{
	DPRINT((HERE("FEND: DisplayMessage, msg=%S, eMsg=%S"), &CONV_UNI(message), &CONV_UNI1(extMessage)));
	TPtrC8	text8((TUint8*) (message));;
	sTextBufer.Copy(text8);
	sFrontendProcessor->DisplayMessageL(sTextBufer);
}

void DisplayError P((String message, int error))
{
	DPRINT((HERE("FEND: DisplayError: %S, err=%d"), &CONV_UNI(message), error));
	TPtrC8	text8((TUint8*) (message));;
	sTextBufer.Copy(text8);
	sFrontendProcessor->DisplayMessageL(sTextBufer);
}

void DisplayMoveError P((String message))
{
	DPRINT((HERE("FEND: DisplayMoveError: %S"), &CONV_UNI(message)));
	TPtrC8	text8((TUint8*) (message));;
	sTextBufer.Copy(text8);
	sFrontendProcessor->DisplayMessageL(sTextBufer);
}


/* If status == 0, we are exiting with a benign message, not an error */
void DisplayFatalError P((String message, int error, int status))
{
	TPtrC8	text8((TUint8*) (message));;
	sTextBufer.Copy(text8);
	sTextBufer.AppendFormat(_L(" err=%d, status=%d"), error, status);
	RDebug::Print(_L("FATAL: %S"), &sTextBufer);
//	sFrontendProcessor->DisplayMessageL(sTextBufer);
//	DEBUGGER();
}


void DisplayInformation P((String message))
{
	TPtrC8	text8((TUint8*) (message));;
	sTextBufer.Copy(text8);
	sFrontendProcessor->DisplayMessageL(sTextBufer);
}

void AskQuestion P((String title, String question, String replyPrefix, ProcRef pr))
{
	DEBUGGER();
}


void DisplayIcsInteractionTitle P((String title))
{
	DEBUGGER();
}

void DrawPosition P((int fullRedraw, Board board))
{
		/*
		 * Hmm, Board is an int[8][8] defined within extern "C"{}.
		 * C++ deals slightly differently with multi-dimensional arrays, so I've defined
		 * TBoard to be an int[8][8] in C++ code, so all .cpp files must use that.
		 */
	DPRINT((HERE("FEND: DrawPosition")));
	if (board)
	{
		TBoard	b;
		Mem::Copy(b, board, sizeof(TBoard));
		sFrontendProcessor->DrawPositionL(fullRedraw, &b);
	}
	else
	{
		sFrontendProcessor->DrawPositionL(fullRedraw, NULL);
	}
}

void ResetFrontEnd P((void))
{
	DPRINT((HERE("FEND: ResetFrontEnd")));
	sFrontendProcessor->ResetFrontEndL();
}


void CommentPopUp P((String title, String comment))
{
	DPRINT((HERE("FEND: CommentPopUp")));
	TPtrC8	title8((TUint8*) (title));
	sTextBufer.Copy(title8);
	TPtrC8	comment8((TUint8*) (comment));
	sTextBufer1.Copy(comment8);
	sFrontendProcessor->CommentPopUpL(sTextBufer, sTextBufer1);
}


void CommentPopDown P((void))
{
	DPRINT((HERE("FEND: CommentPopDown")));
	sFrontendProcessor->CommentPopDown();
}


void EditCommentPopUp P((int index, String title, String text))
{
	DPRINT((HERE("FEND: EditCommentPopUp")));
	TPtrC8	title8((TUint8*) (title));
	sTextBufer.Copy(title8);

	sTextBufer1 = _L("");
	if (text)
	{
		TPtrC8	text8((TUint8*) (text));
		sTextBufer1.Copy(text8);
	}
	sFrontendProcessor->EditCommentPopUpL(index, sTextBufer, sTextBufer1);
}


void RingBell P((void))
{
	DPRINT((HERE("FEND: RingBell - TODO")));
}


void PlayIcsWinSound P((void))
{
	DEBUGGER();
}


void PlayIcsLossSound P((void))
{
	DEBUGGER();
}


void PlayIcsDrawSound P((void))
{
	DEBUGGER();
}


void PlayIcsUnfinishedSound P((void))
{
	DEBUGGER();
}


void PlayAlarmSound P((void))
{
	DEBUGGER();
}


void EchoOn P((void))
{
	DEBUGGER();
}


void EchoOff P((void))
{
	DEBUGGER();
}


void Raw P((void))
{
	// does not appear to be used in Winboard
	DEBUGGER();
}


void Colorize P((ColorClass cc, int continuation))
{
	DEBUGGER();
}


char *UserName P((void))
{
	DPRINT((HERE("FEND: UserName - TODO")));
	return "User";
}


char *HostName P((void))
{
	DPRINT((HERE("FEND: HostName - TODO")));
	return "locahost";
}



static CPeriodic*	sClockTimer = NULL;

int ClockTimerRunning P((void))
{
//	DPRINT((HERE("FEND: ClockTimerRunning")));
	TBool	isRunning = EFalse;
	if (sClockTimer)
		isRunning = sClockTimer->IsActive();
	return isRunning;
}


int StopClockTimer P((void))
{
//	DPRINT((HERE("FEND: StopClockTimer")));

	TBool	wasRunning = EFalse;
	if (sClockTimer)
	{
		if (sClockTimer->IsActive())
			wasRunning = EFalse;
		sClockTimer->Cancel();
	}
	return wasRunning;
}


TInt ClockTimerCallbackFunc(TAny* aPtr)
{
	DecrementClocks();	// call back into backend
	return 0;
}


void StartClockTimer P((long millisec))
{
//	DPRINT((HERE("FEND: StartClockTimer, ms=%d"), millisec));
	StopClockTimer();
	if (!sClockTimer)
	{
		TRAPD(err, sClockTimer = CPeriodic::NewL(CActive::EPriorityHigh));
	}
	if (sClockTimer)
		sClockTimer->Start(millisec * 1000, 0, TCallBack(ClockTimerCallbackFunc));
}


void DisplayWhiteClock P((long timeRemaining, int highlight))
{
//	DPRINT((HERE("FEND: DisplayWhiteClock: remain=%d, highlight=%d"), timeRemaining, highlight));
	TPtrC8	text8((TUint8*) TimeString(timeRemaining));
	sTextBufer.Copy(text8);
	sFrontendProcessor->UpdateTimeClockL(ETrue, sTextBufer, highlight);
}


void DisplayBlackClock P((long timeRemaining, int highlight))
{
//	DPRINT((HERE("FEND: DisplayBlackClock: remain=%d, highlight=%d"), timeRemaining, highlight));
	TPtrC8	text8((TUint8*) TimeString(timeRemaining));
	sTextBufer.Copy(text8);
	sFrontendProcessor->UpdateTimeClockL(EFalse, sTextBufer, highlight);
}


TInt LoadGameTimerCallbackFunc(TAny* aPtr)
{
	AutoPlayGameLoop(); /* call into back end */
	return 0;
}


static CPeriodic*	sLoadGameTimer = NULL;

int LoadGameTimerRunning P((void))
{
	DPRINT((HERE("FEND: LoadGameTimerRunning")));

	TBool	isRunning = EFalse;
	if (sLoadGameTimer)
		isRunning = sLoadGameTimer->IsActive();
	return isRunning;
}


int StopLoadGameTimer P((void))
{
	DPRINT((HERE("FEND: StopLoadGameTimer")));

	TBool	wasRunning = EFalse;
	if (sLoadGameTimer)
	{
		if (sLoadGameTimer->IsActive())
			wasRunning = EFalse;
		sLoadGameTimer->Cancel();
	}
	return wasRunning;
}


void StartLoadGameTimer P((long millisec))
{
	DPRINT((HERE("FEND: StartLoadGameTimer: ms=%d"), millisec));

	StopLoadGameTimer();
	if (!sLoadGameTimer)
	{
		TRAPD(err, sLoadGameTimer = CPeriodic::NewL(CActive::EPriorityHigh));
	}
	if (sLoadGameTimer)
		sLoadGameTimer->Start(millisec * 1000, 0, TCallBack(LoadGameTimerCallbackFunc));
}

void AutoSaveGame P((void))
{
	DEBUGGER();
}

DelayedEventCallback	sDelayedEventCallback = NULL;
static CPeriodic*			sDelayedEventTimer = NULL;
static TInt DelayedEventCallbackFunc(TAny*)
{
	if (sDelayedEventCallback)
		sDelayedEventCallback();
	CancelDelayedEvent();
	return 0;
}
void ScheduleDelayedEvent P((DelayedEventCallback cb, long millisec))
{
	DPRINT((HERE("FEND: ScheduleDelayedEvent: ms=%d"), millisec));

	CancelDelayedEvent();
	if (!sDelayedEventTimer)
	{
		TRAPD(err, sDelayedEventTimer = CPeriodic::NewL(CActive::EPriorityHigh));
		sDelayedEventCallback = cb;
	}
	if (sDelayedEventTimer)
		sDelayedEventTimer->Start(millisec * 1000, 0, DelayedEventCallbackFunc);
}

DelayedEventCallback GetDelayedEvent P((void))
{
	DPRINT((HERE("FEND: GetDelayedEvent")));
	return sDelayedEventCallback;
}

void CancelDelayedEvent P((void))
{
	DPRINT((HERE("FEND: CancelDelayedEvent")));
	sDelayedEventCallback = NULL;
	if (sDelayedEventTimer)
		sDelayedEventTimer->Cancel();
}

int StartChildProcess P((char *cmdLine, char *dir, ProcRef *pr))
{
	DPRINT((HERE("FEND: StartChildProcess: cmdLine=%S, dis=%S"), &CONV_UNI(cmdLine), &CONV_UNI1(dir)));

	CChildProcChessEngine*	childProcChessEngine = NULL;
	TRAPD(err, childProcChessEngine = CChildProcChessEngine::NewL());

	*pr = (ProcRef) childProcChessEngine;
	return err;
}

void DestroyChildProcess P((ProcRef pr, int/*boolean*/ signal))
{
	DEBUGGER();
	CChildProcBase*	proc = (CChildProcBase*) pr;
	if (proc)
	{
		proc->DestroyChildProcess(signal);
	}
}

void InterruptChildProcess P((ProcRef pr))
{
	DPRINT((HERE("FEND: InterruptChildProcess")));
	CChildProcBase*	proc = (CChildProcBase*) pr;
	if (proc)
		proc->InterruptChildProcess();
}

int OpenTelnet P((char *host, char *port, ProcRef *pr))
{
	DEBUGGER();
	return 0;
}

int OpenTCP P((char *host, char *port, ProcRef *pr))
{
	DEBUGGER();
	return 0;
}

int OpenCommPort P((char *name, ProcRef *pr))
{
	DEBUGGER();
	return 0;
}

int OpenLoopback P((ProcRef *pr))
{
	DEBUGGER();
	return 0;
}

int OpenRcmd P((char *host, char *user, char *cmd, ProcRef *pr))
{
	DEBUGGER();
	return 0;
}


InputSourceRef AddInputSource P((ProcRef pr, int lineByLine,
				 InputCallback func, VOIDSTAR closure))
{
	DPRINT((HERE("FEND: AddInputSource")));

	CChildProcBase*	proc = (CChildProcBase*) pr;
	if (proc)
		proc->AddInputSource (lineByLine, func, closure);

	return proc;
}

void RemoveInputSource P((InputSourceRef isr))
{
	DEBUGGER();
	CChildProcBase*	proc = (CChildProcBase*) isr;
	if (proc)
		proc->RemoveInputSource();
}

/* pr == NoProc means the local display */
int OutputToProcess P((ProcRef pr, char *message, int count, int *outError))
{
	DPRINT((HERE("FEND: OutputToProcess: %S"), &CONV_UNI(message)));

	*outError = KErrUnknown;	// whatever
	CChildProcBase*	proc = (CChildProcBase*) pr;
	if (proc)
		*outError = proc->OutputToProcessDelayed ((TText8*) message, count);
	DPRINT((HERE("FEND: OutputToProcess: done")));

	return (*outError == KErrNone) ? count : 0;
}

int OutputToProcessDelayed P((ProcRef pr, char *message, int count,
			      int *outError, long msdelay))
{
	DPRINT((HERE("FEND: OutputToProcessDelayed: %S"), &CONV_UNI(message)));

	*outError = KErrUnknown;	// whatever
	CChildProcBase*	proc = (CChildProcBase*) pr;
	if (proc)
		*outError = proc->OutputToProcessDelayed ((TText8*) message, count, msdelay);
	DPRINT((HERE("FEND: OutputToProcessDelayed: done")));

	return (*outError == KErrNone) ? count : 0;
}

void CmailSigHandlerCallBack P((InputSourceRef isr, VOIDSTAR closure,
				char *buf, int count, int error))
{
	DEBUGGER();
}


void GameListPopUp P((FILE *fp, char *filename))
{
//	DPRINT((HERE("FEND: GameListPopUp - TODO: %S"), &CONV_UNI(filename)));
	// nothing to be done, solved differently
	DEBUGGER();	// shouldn't happen
}

void GameListPopDown P((void))
{
//	DPRINT((HERE("FEND: GameListPopDown - TODO")));
	// nothing to be done, solved differently
}

void GameListHighlight P((int index))
{
	DPRINT((HERE("FEND: GameListHighlight - TODO: index=%d"), index));
}

void GameListDestroy P((void))
{
	DPRINT((HERE("FEND: GameListDestroy - TODO")));
}


void EditTagsPopUp P((char *tags))
{
//	DPRINT((HERE("FEND: EditTagsPopUp - TODO, tags=%S"), &CONV_UNI(tags)));
	// nothing to be done tags handled by App
}

void TagsPopUp P((char *tags, char *msg))
{
//	DPRINT((HERE("FEND: TagsPopUp - TODO, tags=%S, msg=%S"), &CONV_UNI(tags), &CONV_UNI1(msg)));
	// nothing to be done tags handled by App
}

void TagsPopDown P((void))
{
//	DPRINT((HERE("FEND: TagsPopDown - TODO")));
	// nothing to be done tags handled by App
}

void ICSInitScript P((void))
{
	DEBUGGER();
}

void StartAnalysisClock P((void))
{
	DEBUGGER();
}

void AnalysisPopUp P((char *title, char *label))
{
	DPRINT((HERE("FEND: AnalysisPopUp - not supported by GNUCHESS")));
}

void AnalysisPopDown P((void))
{
	DPRINT((HERE("FEND: AnalysisPopDown - not supported by GNUCHESS")));
}


void SetHighlights P((int fromX, int fromY, int toX, int toY))
{
	DPRINT((HERE("FEND: SetHighlights, from=(%d, %d), to=(%d, %d)"), fromY, fromX, toX, toY));
	sFrontendProcessor->SetHighlightsL(TSquare(fromY, fromX), TSquare(toY, toX));
}

void ClearHighlights P((void))
{
	DPRINT((HERE("FEND: ClearHighlights")));
	sFrontendProcessor->ClearHighlightsL();
}

void SetPremoveHighlights P((int fromX, int fromY, int toX, int toY))
{
	DEBUGGER();
}

void ClearPremoveHighlights P((void))
{
	DPRINT((HERE("FEND: ClearPremoveHighlights")));
	sFrontendProcessor->ClearPremoveHighlightsL();
}

void ShutDownFrontEnd P((void))
{
	DEBUGGER();
}

void BoardToTop P((void))
{
	DEBUGGER();
}

void AnimateMove P((Board board, int fromX, int fromY, int toX, int toY))
{
	DPRINT((HERE("FEND: AnimateMove - TODO")));
}

void HistorySet P((char movelist[][2*MOVE_LEN],
		   int first, int last, int current))
{
	DPRINT((HERE("FEND: HistorySet - not supported")));
	 /* Currently not implemented in WinBoard */
}

void FreezeUI P((void))
{
	DPRINT((HERE("FEND: FreezeUI")));
}

void ThawUI P((void))
{
	DEBUGGER();
}




char* programVersion = "EChess 1.0";
int tinyLayout = 1;
int smallLayout = 0;
char* programName = "EChess";

/****************************************
 *
 * Missing standard C functions
 *
 ****************************************/

FILE* popen(const char* name, const char* mode)
{
	DEBUGGER();
	return NULL;
}

int pclose(FILE* file)
{
	DEBUGGER();
	return 0;
}
