/*******************************************************************
 *
 *	File:		EGnuChessMiscCtls.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

// Draw is a private method in CCoeControl, we make ourselves a friend in order
// to call Draw on a CCoeControl
#define IgnoreEventsUntilNextPointerUp() IgnoreEventsUntilNextPointerUp();friend class CCoeControlContainer;
#include <coecntrl.h>

#include "EGnuChessMiscCtls.h"

#include <eikenv.h>
#include <eiklabel.h>
#include "debug.h"


template<class T>
void RemoveFromPointerArrayList(CArrayPtrFlat<T>& aList, T* aT)
{
	TInt	count = aList.Count();
	for (TInt i=count-1 ; i>=0 ; i--)
		if ((aList)[i] == aT)
			aList.Delete(i);
}


/***********************************************************
 *
 * CCoeControlContainer
 *
 ***********************************************************/

CCoeControlContainer::~CCoeControlContainer()
{
	delete iChildren;
	delete iDontBlankChildren;
	iBlankChildrenRegion.Clear();
}

void CCoeControlContainer::DestroyChildren()
{
	if (iChildren)
	{
		TInt count = iChildren->Count();
		for (TInt i = count-1 ; i>=0 ; i--)
			delete iChildren->At(i);
	}
}

void CCoeControlContainer::ConstructL()
{
	SetComponentsToInheritVisibility(ETrue);

	iChildren = new(ELeave)CArrayPtrFlat<CCoeControl>(5);
	iDontBlankChildren = new(ELeave)CArrayPtrFlat<CCoeControl>(5);
}

TBool CCoeControlContainer::ChildNeedsBlanking(CCoeControl* aChild)
{
	TBool	childNeedsBlanking = ETrue;
	TInt count = iDontBlankChildren->Count();
	for (TInt i = 0 ; i<count ; i++)
	{
		if (iDontBlankChildren->At(i) == aChild)
		{
			childNeedsBlanking = EFalse;
			break;
		}
	}
	return childNeedsBlanking;
}


void CCoeControlContainer::AddChildL(CCoeControl* aChild, TBool aBlankChild)
{
	iChildren->AppendL(aChild);
	if (!aBlankChild)
		iDontBlankChildren->AppendL(aChild);
}

void CCoeControlContainer::RemoveChild(CCoeControl* aChild)
{
	RemoveFromPointerArrayList(*iChildren, aChild);
	RemoveFromPointerArrayList(*iDontBlankChildren, aChild);
}

void CCoeControlContainer::LayoutChildren()
{
	DoLayoutChildren();
	SetRegion();
}


CCoeControl* CCoeControlContainer::ComponentControl(TInt aIndex) const
{
	return iChildren->At(aIndex);
}

TInt CCoeControlContainer::CountComponentControls() const
{
	return iChildren->Count();
}

void CCoeControlContainer::Draw(const TRect& aRect) const
{
		/*
		 * clear all but the Rect taken by our children
		 */
	CWindowGc& gc = SystemGc();
	gc. SetClippingRegion(iBlankChildrenRegion);
	gc.Clear(aRect);
	gc. CancelClippingRegion();
}

void CCoeControlContainer::SetRegion()
{
	TInt	count = iDontBlankChildren->Count();
	iBlankChildrenRegion.Clear();
	iBlankChildrenRegion = RRegion(Rect(), count);
	for (TInt i = 0 ; i<count ; i++)
		iBlankChildrenRegion.SubRect(iDontBlankChildren->At(i)->Rect(), NULL);
}

void CCoeControlContainer::DrawChildNow(CCoeControl* aChild)
{
	TRect	rect = aChild->Rect();
	
	Window().Invalidate(rect);
	ActivateGc();
	Window().BeginRedraw(rect);
	if (ChildNeedsBlanking(aChild))
		SystemGc().Clear(rect);
	Draw(rect);
	aChild->Draw(rect);
	Window().EndRedraw();
	DeactivateGc();
	iEikonEnv->WsSession().Flush();
}


void CCoeControlContainer::DrawAreaNow(const TRect& aRect)
{
	Window().Invalidate(aRect);
	HandleRedrawEvent(aRect);
}

/***********************************************************
 *
 * CBitmappedCoeControl
 *
 ***********************************************************/

void CBitmappedCoeControl::DrawControl()
{
	if (iBitmap)
	{
		iBitmapContext->Clear();

		// DrawControl will draw relative to its Position().
		// when drawing to the bitmap gc, Position() should be (0,0)
		TPoint	position = iPosition;
		iPosition = TPoint(0,0);
		DrawControl(*iBitmapContext, *iBitmapDevice);
		iPosition = position;
	}
}

void CBitmappedCoeControl::SizeChanged()
{
		/*
		 * Destroy old bitmap and try to create a new one.
		 * If the new one is created, call DrawControl which will
		 * draw it that bitmap (we'll wait until we get a redraw
		 * to blit it to screen); if it can't be created we do nothing
		 * (from the redraw we call DrawControl with the window's gc)
		 */
	DestroyBitmap();
	TRAPD(err, CreateBitmapL(Size()));
	if (err)
		DestroyBitmap();
	else
		DrawControl();
}

void CBitmappedCoeControl::DestroyBitmap()
{
	delete iBitmapContext;
	iBitmapContext = NULL;
	delete iBitmapDevice;
	iBitmapDevice = NULL;
	delete iBitmap;
	iBitmap = NULL;
}

void CBitmappedCoeControl::CreateBitmapL(const TSize& aSize)
{
	iBitmap = new (ELeave) CFbsBitmap;
	iBitmap->Create(aSize, iEikonEnv->ScreenDevice()->DisplayMode());

	iBitmapDevice = CFbsBitmapDevice::NewL(iBitmap);
	iBitmapDevice->CreateContext(iBitmapContext);
}

void CBitmappedCoeControl::Draw(const TRect& aRect) const
{
    CWindowGc& gc=SystemGc();

	if (iBitmap)
		gc.BitBlt(aRect.iTl, iBitmap, TRect(aRect.iTl - Position(), aRect.Size()));
	else
	{
		CWsScreenDevice* screenDevice = iCoeEnv->ScreenDevice();
		DrawControl(gc, *screenDevice);
	}
}


/***********************************************************
 *
 * CBitmappedLabel
 *
 ***********************************************************/

CBitmappedLabel::CBitmappedLabel()
{
	iSystemFont = iCoeEnv->NormalFont();
	iBgColor = KRgbWhite;
}

CBitmappedLabel::~CBitmappedLabel()
{
	delete iText;
	ReleaseUserFont();
}


void CBitmappedLabel::ReleaseUserFont()
{
	if (iUserFont)
	{
		iCoeEnv->ScreenDevice()->ReleaseFont(iUserFont);
		iUserFont = NULL;
	}
}

const CFont* CBitmappedLabel::Font() const
{
	return iUserFont ? iUserFont : iSystemFont;
}

TInt CBitmappedLabel::UseFont(const TDesC& aFontName, TInt aFontSize)
{
	ReleaseUserFont();

	TFontSpec fontSpec(aFontName, aFontSize);
	return iCoeEnv->ScreenDevice()->GetNearestFontInTwips(iUserFont, fontSpec);
}


void CBitmappedLabel::SetTextL(const TDesC& aText)
{
	HBufC*	text = HBufC::NewLC(aText.Length());
	CleanupStack::Pop();
	text->Des().Copy(aText);
	delete iText;
	iText = text;

	CBitmappedCoeControl::DrawControl();
}

static TInt FontTextCount(const TDesC& aText, const CFont& aFont, TInt aWidth)
{
	TInt pos = aFont.TextCount(aText, aWidth);
	if ((aText.Length() > pos))
	{
		if (aText[pos+1] != TChar(' '))
		{
			TInt pos1 = aText.Left(pos).LocateReverse(TChar(' '));
			if ((pos1 > 0) && (pos1<pos))
				pos = pos1+1;
		}
	}
	return pos;
}

void CBitmappedLabel::DrawControl(CBitmapContext& aBitmapContext, CBitmapDevice& /*aBitmapDevice*/) const
{
		// make sure we have text
	if (!iText)
		return;

	const CFont*	font = iUserFont ? iUserFont : iSystemFont;
	TRect			fullRect = Rect();

	aBitmapContext.UseFont(font);
	aBitmapContext.SetBrushColor(iBgColor);
	aBitmapContext.Clear(fullRect);

	aBitmapContext.SetPenColor(KRgbBlack);
	aBitmapContext.SetPenSize(TSize(1,1));


	TInt	lineHeight = font->HeightInPixels()+1;
	TInt	lineWidth = fullRect.Size().iWidth;
	TRect	lineRect(fullRect.iTl, TSize(lineWidth, lineHeight));

	TPtrC	p(*iText);
	TBool	done = EFalse;
	while (!done)
	{
//		TInt len = font->TextCount(p, lineWidth);
		TInt len = FontTextCount(p, *font, lineWidth);
		aBitmapContext.DrawText(p.Left(len), lineRect, font->AscentInPixels(), CGraphicsContext::ELeft);

		lineRect.Move(0, lineHeight);
		if (len < p.Length())
			 p.Set(p.Right(p.Length() - len));
		else
			done = ETrue;
		if (lineRect.iBr.iY > fullRect.iBr.iY)
			done = ETrue;
	}
}
