/*******************************************************************
 *
 *	File:		EGnuChessModel.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2002, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "EGnuChessModel.h"
#include "EGnuChessModelObserver.h"
#include "EGnuChessDocument.h"
#include <eikenv.h>
#include <Echess.rsg>
#include <string.h>
#include <stdlib.h>

CEGnuChessModel* CEGnuChessModel::NewL(CEGnuChessDocument& aDocument)
{
	CEGnuChessModel* self=new (ELeave) CEGnuChessModel(aDocument);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);
	return self;
}

CEGnuChessModel::~CEGnuChessModel()
{
}


CEGnuChessModel::CEGnuChessModel(CEGnuChessDocument& aDocument) : iDocument(aDocument)
{
}


void CEGnuChessModel::ConstructL()
{
	iEGnuChessOptions.Load(CEikonEnv::Static()->FsSession());
	InitAppData();
	SetTimeControl();
	SetFrontendProcessor(this);
}


void CEGnuChessModel::AddModelBoardObserver(MEGnuChessModelBoardObserver& aModelBoardObserver)
{
	for (TInt i=0 ; i<KMaxObservers ; i++)
	{
		if (!iModelBoardObservers[i])
		{
			iModelBoardObservers[i] = &aModelBoardObserver;
			break;
		}
	}
}

void CEGnuChessModel::AddModelGameObserver(MEGnuChessModelGameObserver& aModelGameObserver)
{
	for (TInt i=0 ; i<KMaxObservers ; i++)
	{
		if (!iModelGameObservers[i])
		{
			iModelGameObservers[i] = &aModelGameObserver;
			break;
		}
	}
}

void CEGnuChessModel::SetModelTimeObserver(MEGnuChessModelTimeObserver& aModelTimeObserver)
{
	iModelTimeObserver = &aModelTimeObserver;
}


void CEGnuChessModel::StartBackEnd()
{
	InitBackEnd1();
	InitBackEnd2();
}


void CEGnuChessModel::InitAppData()
{
	  /* Initialize to defaults */
	appData.movesPerSession = MOVES_PER_SESSION;
	appData.initString = INIT_STRING;
	appData.secondInitString = INIT_STRING;
	appData.firstComputerString = "";
	appData.secondComputerString = "";
	appData.firstChessProgram = FIRST_CHESS_PROGRAM;
	appData.secondChessProgram = SECOND_CHESS_PROGRAM;
	appData.firstPlaysBlack = FALSE;
	appData.noChessProgram = FALSE;
	appData.firstHost = FIRST_HOST;
	appData.secondHost = SECOND_HOST;
	appData.firstDirectory = FIRST_DIRECTORY;
	appData.secondDirectory = SECOND_DIRECTORY;
	appData.bitmapDirectory = "";
	appData.remoteShell = REMOTE_SHELL;
	appData.remoteUser = "";
	appData.timeDelay = TIME_DELAY;
	appData.timeControl = NULL;
	appData.timeIncrement = TIME_INCREMENT;
	appData.icsActive = FALSE;
	appData.icsHost = "";
	appData.icsPort = ICS_PORT;
	appData.icsCommPort = ICS_COMM_PORT;
	appData.icsLogon = ICS_LOGON;
	appData.icsHelper = "";
	appData.useTelnet = FALSE;
	appData.telnetProgram = TELNET_PROGRAM;
	appData.gateway = "";
	appData.loadGameFile = "";
	appData.loadGameIndex = 0;
	appData.saveGameFile = "";
	appData.autoSaveGames = FALSE;
	appData.loadPositionFile = "";
	appData.loadPositionIndex = 1;
	appData.savePositionFile = "";
	appData.matchMode = FALSE;
	appData.matchGames = 0;
	appData.monoMode = FALSE;
	appData.debugMode = FALSE;
	appData.clockMode = TRUE;
	appData.Iconic = FALSE; /*unused*/
	appData.searchTime = "";
	appData.searchDepth = 0;
	appData.showCoords = FALSE;
	appData.ringBellAfterMoves = TRUE; /*obsolete in WinBoard*/
	appData.autoCallFlag = FALSE;
	appData.flipView = FALSE;
	appData.autoFlipView = TRUE;
	appData.cmailGameName = "";
	appData.alwaysPromoteToQueen = FALSE;
	appData.oldSaveStyle = FALSE;
	appData.quietPlay = FALSE;
	appData.showThinking = FALSE;
	appData.ponderNextMove = TRUE;
	appData.periodicUpdates = TRUE;
	appData.popupExitMessage = TRUE;
	appData.popupMoveErrors = FALSE;
	appData.autoObserve = FALSE;
	appData.autoComment = FALSE;
	appData.animate = TRUE;
	appData.animSpeed = 10;
	appData.animateDragging = TRUE;
	appData.highlightLastMove = TRUE;
	appData.getMoveList = TRUE;
	appData.testLegality = TRUE;
	appData.premove = TRUE;
	appData.premoveWhite = FALSE;
	appData.premoveWhiteText = "";
	appData.premoveBlack = FALSE;
	appData.premoveBlackText = "";
	appData.icsAlarm = TRUE;
	appData.icsAlarmTime = 5000;
	appData.autoRaiseBoard = TRUE;
	appData.localLineEditing = TRUE;
	appData.colorize = TRUE;
	appData.reuseFirst = TRUE;
	appData.reuseSecond = TRUE;
	appData.blindfold = FALSE;

	appData.initialMode = "";
	appData.variant = "normal";
	appData.firstProtocolVersion = PROTOVER;
	appData.secondProtocolVersion = PROTOVER;
	appData.showButtonBar = TRUE;
#ifdef ZIPPY
	appData.zippyTalk = ZIPPY_TALK;
	appData.zippyPlay = ZIPPY_PLAY;
	appData.zippyLines = ZIPPY_LINES;
	appData.zippyPinhead = ZIPPY_PINHEAD;
	appData.zippyPassword = ZIPPY_PASSWORD;
	appData.zippyPassword2 = ZIPPY_PASSWORD2;
	appData.zippyWrongPassword = ZIPPY_WRONG_PASSWORD;
	appData.zippyAcceptOnly = ZIPPY_ACCEPT_ONLY;
	appData.zippyUseI = ZIPPY_USE_I;
	appData.zippyBughouse = ZIPPY_BUGHOUSE;
	appData.zippyNoplayCrafty = ZIPPY_NOPLAY_CRAFTY;
	appData.zippyGameEnd = ZIPPY_GAME_END;
	appData.zippyGameStart = ZIPPY_GAME_START;
	appData.zippyAdjourn = ZIPPY_ADJOURN;
	appData.zippyAbort = ZIPPY_ABORT;
	appData.zippyVariants = ZIPPY_VARIANTS;
	appData.zippyMaxGames = ZIPPY_MAX_GAMES;
	appData.zippyReplayTimeout = ZIPPY_REPLAY_TIMEOUT;
#endif
}

TBool CEGnuChessModel::IsPromotion(const TSquare& aSquare)
{
	return (::IsPromotion(iHighlights[0].iCol, iHighlights[0].iRow, aSquare.iCol, aSquare.iRow) != 0);
}

TBool CEGnuChessModel::IsOKToStartUserMove(const TSquare& aSquare)
{
	return (::OKToStartUserMove(aSquare.iCol, aSquare.iRow) != 0);
}

void CEGnuChessModel::UserMoveEvent(const TSquare& aSquare, TInt aPromoChar)
{
	::UserMoveEvent(iHighlights[0].iCol, iHighlights[0].iRow, aSquare.iCol, aSquare.iRow, aPromoChar);
}

void CEGnuChessModel::SetHighlight(const TSquare& aSquare)
{
	::SetHighlights(aSquare.iCol, aSquare.iRow, -1, -1);
}



/**********************************************
 *
 * FRONTEND PROCESSOR
 *
 **********************************************/

void CEGnuChessModel::ModeHighlightL()
{
	for (TInt i=0 ; i<KMaxObservers ; i++)
	{
		if (!iModelGameObservers[i])
			break;
		iModelGameObservers[i]->ModeHighlightL();
	}
}


void CEGnuChessModel::ResetFrontEndL()
{
	iFileName = _L("");
	iFileName8 = _L8("");
	for(TInt row=0 ; row<8 ; row++)
		for(TInt col=0 ; col<8 ; col++)
			iBoard[row][col] = iPrevBoard[row][col] = ClearBoard;
}

void CEGnuChessModel::ClearHighlightsL()
{
	iHighlights[0].iRow = iHighlights[0].iCol = -1;
	iHighlights[1].iRow = iHighlights[1].iCol = -1;

	for (TInt i=0 ; i<KMaxObservers ; i++)
	{
		if (!iModelBoardObservers[i])
			break;
		iModelBoardObservers[i]->DrawHighlightsL();
	}
}


void CEGnuChessModel::ClearPremoveHighlightsL()
{
	iPRemoveHighlights[0].iRow = iPRemoveHighlights[0].iCol = -1;
	iPRemoveHighlights[1].iRow = iPRemoveHighlights[1].iCol = -1;
}

void CEGnuChessModel::DrawPositionL(TInt aFullRedraw, TBoard* aBoard)
{
	if (aBoard)
	{
		Mem::Copy(iPrevBoard, iBoard, sizeof(TBoard));
		Mem::Copy(iBoard, aBoard, sizeof(TBoard));
	}

	for (TInt i=0 ; i<KMaxObservers ; i++)
	{
		if (iModelBoardObservers[i])
			iModelBoardObservers[i]->RedrawL(aFullRedraw);
		else
			break;
	}
}

void CEGnuChessModel::SetHighlightsL(const TSquare& aSquareFrom, const TSquare& aSquareTo)
{
	iHighlights[0] = aSquareFrom;
	iHighlights[1] = aSquareTo;

	for (TInt i=0 ; i<KMaxObservers ; i++)
	{
		if (!iModelBoardObservers[i])
			break;
		iModelBoardObservers[i]->DrawHighlightsL();
	}
}


void CEGnuChessModel::UserThinksL()
{
	iUserThinks = ETrue;
	for (TInt i=0 ; i<KMaxObservers ; i++)
	{
		if (!iModelGameObservers[i])
			break;
		iModelGameObservers[i]->UserThinksL();
	}
}

void CEGnuChessModel::MachineThinksL()
{
	iUserThinks = EFalse;
	for (TInt i=0 ; i<KMaxObservers ; i++)
	{
		if (!iModelGameObservers[i])
			break;
		iModelGameObservers[i]->MachineThinksL();
	}
}

void CEGnuChessModel::DisplayMessageL(const TDesC& aMessage)
{
	for (TInt i=0 ; i<KMaxObservers ; i++)
	{
		if (!iModelGameObservers[i])
			break;
		iModelGameObservers[i]->DisplayMessageL(aMessage);
	}
}

void CEGnuChessModel::UpdateTimeClockL(TBool aWhite, const TDesC& aTimeString, TBool aHighlight)
{
	if (iModelTimeObserver)
		iModelTimeObserver->UpdateTimeClockL(aWhite, aTimeString, aHighlight);
}

void CEGnuChessModel::CommentPopUpL(const TDesC& aTitle, const TDesC& aComment)
{
	for (TInt i=0 ; i<KMaxObservers ; i++)
	{
		if (!iModelGameObservers[i])
			break;
		iModelGameObservers[i]->CommentPopUpL(aTitle, aComment);
	}
}

void CEGnuChessModel::CommentPopDown()
{
	for (TInt i=0 ; i<KMaxObservers ; i++)
	{
		if (!iModelGameObservers[i])
			break;
		iModelGameObservers[i]->CommentPopDown();
	}
}

void CEGnuChessModel::EditCommentPopUpL(TInt aIndex, const TDesC& aTitle, const TDesC& aText)
{
	for (TInt i=0 ; i<KMaxObservers ; i++)
	{
		if (!iModelGameObservers[i])
			break;
		iModelGameObservers[i]->EditCommentPopUpL(aIndex, aTitle, aText);
	}
}


void CEGnuChessModel::SaveGame()
{
	iFileName8.Copy(iFileName);
	const char* filename = (const char*) iFileName8.PtrZ();
	FILE* f =     f = fopen(filename, "w");
	if (f)
	{
		::SaveGame(f, 0, NULL);
		fclose(f);
		User::InfoPrint(_L("Game saved"));
	}
}

void CEGnuChessModel::LoadGame(const TDesC& aFileName, TInt& aNoOfGames)
{
	iFileName = aFileName;
	iFileName8.Copy(iFileName);
	iFile  = fopen((const char*)iFileName8.PtrZ(), "r");
	aNoOfGames = -1;
	if (iFile)
	{
		TInt error = ::GameListBuild(iFile);
		if (error) 
		{
			::DisplayError("Cannot build game list", error);
		} 
		else if (!ListEmpty(&gameList) &&
				((ListGame *) gameList.tailPred)->number > 1) 
		{
//			::GameListPopUp(f, filename);
			aNoOfGames = ((ListGame *) gameList.tailPred)->number;
			return;
		}
		::GameListDestroy();
		SelectedGame(1, EFalse);	// don't use list
	}
}


void CEGnuChessModel::SelectedGame(TInt aGameNo, TBool aUseList)
{
	::LoadGame(iFile, aGameNo, (char*)iFileName8.PtrZ(), aUseList);
}


const TEGnuChessOptions& CEGnuChessModel::ChessOptions()
{
	return iEGnuChessOptions;
}

void CEGnuChessModel::SetChessOptions(const TEGnuChessOptions& aOptions)
{
	iEGnuChessOptions = aOptions;
}


void CEGnuChessModel::SetTimeControl()
{
	if (appData.timeControl)
	{
		free(appData.timeControl);
		appData.timeControl = NULL;
	}
	if (iEGnuChessOptions.iClockSettings.iClockType == TClockSettings::EClockTypeConventional)
	{
		TBuf8<32>	buf;
		buf.Copy(iEGnuChessOptions.iClockSettings.iConventionalTime);
		appData.timeControl = strdup((char*)buf.PtrZ());
		appData.movesPerSession = iEGnuChessOptions.iClockSettings.iConventionalNoOfMoves;
		appData.timeIncrement = -1;
	}
	else
	{
		TBuf8<32>	buf;
		buf.Copy(iEGnuChessOptions.iClockSettings.iIncrementalInitialTime);
		appData.timeControl = strdup((char*)buf.PtrZ());
		appData.movesPerSession = 0;
		appData.timeIncrement = iEGnuChessOptions.iClockSettings.iIncrementalAdditionalTimePerMove;
	}
}

